/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2006 Embedded Artists AB
 *
 * Description:
 *    Sample application that demonstrates how to create processes.
 *
 *****************************************************************************/

#include <general.h>
#include <printf_P.h>
#include <ea_init.h>
#include <lpc246x.h>
#include "adc.h"

#define PROC1_STACK_SIZE  1300
#define INIT_STACK_SIZE   512


/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  T1TCR = 0x02;          //stop and reset timer
  T1PR  = 0x00;          //set prescaler to zero
  T1MR0 = delayInMs * (Fpclk / 1000);
  T1IR  = 0xff;          //reset all interrrupt flags
  T1MCR = 0x04;          //stop timer on match
  T1TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (T1TCR & 0x01)
    ;
}


/*****************************************************************************
 *
 * Description:
 *    The first function to execute 
 *
 ****************************************************************************/
int
main(void)
{
  eaInit();

  printf("\n***************************************************");
  printf("\n*                                                 *");
  printf("\n* Test program for LPC2388 EDU Board...           *");
  printf("\n* (C) Embedded Artists 2008                       *");
  printf("\n*                                                 *");
  printf("\n* Test - Analog input                             *");
  printf("\n*                                                 *");
  printf("\n***************************************************\n");

  initAdc();
  
  for(;;)
  {
  	tU8 message[] = "Ain #0 =     , Ain #1 =     , Ain #2 =     ";
  	tU32 value;

    value = getAnalogueInput(0);
    message[9]  =  (value / 1000)      + '0';
    message[10] = ((value / 100) % 10) + '0';
    message[11] = ((value / 10)  % 10) + '0';
    message[12] =  (value        % 10) + '0';
    if (message[9] == '0')
    {
      message[9] = ' ';
      if (message[10] == '0')
      {
        message[10] = ' ';
        if (message[11] == '0')
          message[11] = ' ';
      }
    }

    value = getAnalogueInput(1);
    message[24] =  (value / 1000)      + '0';
    message[25] = ((value / 100) % 10) + '0';
    message[26] = ((value / 10)  % 10) + '0';
    message[27] =  (value        % 10) + '0';
    if (message[24] == '0')
    {
      message[24] = ' ';
      if (message[25] == '0')
      {
        message[25] = ' ';
        if (message[26] == '0')
          message[26] = ' ';
      }
    }

    value = getAnalogueInput(2);
    message[39] =  (value / 1000)      + '0';
    message[40] = ((value / 100) % 10) + '0';
    message[41] = ((value / 10)  % 10) + '0';
    message[42] =  (value        % 10) + '0';
    if (message[39] == '0')
    {
      message[39] = ' ';
      if (message[40] == '0')
      {
        message[40] = ' ';
        if (message[41] == '0')
          message[42] = ' ';
      }
    }

  printf("\r%s", message);
  delayMs(40);
  }
}
