/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2008 Embedded Artists AB
 *
 * Description:
 *    Sample application that demonstrates how to use rtc.
 *
 *****************************************************************************/

#include <general.h>
#include <printf_P.h>
#include <ea_init.h>
#include <lpc246x.h>

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  T1TCR = 0x02;          //stop and reset timer
  T1PR  = 0x00;          //set prescaler to zero
  T1MR0 = delayInMs * (Fpclk / 1000);
  T1IR  = 0xff;          //reset all interrrupt flags
  T1MCR = 0x04;          //stop timer on match
  T1TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (T1TCR & 0x01)
    ;
}

/*****************************************************************************
 *
 * Description:
 *    The first function to execute 
 *
 ****************************************************************************/
int
main(void)
{
  IODIR1 |= 0x38000000;
  IOCLR1  = 0x38000000;
  PINSEL3 = 0;

  eaInit();
  
  printf("\n***************************************************");
  printf("\n*                                                 *");
  printf("\n* Test program for LPC24xx EDU Board...           *");
  printf("\n* (C) Embedded Artists 2008                       *");
  printf("\n*                                                 *");
  printf("\n* Test - Joystick                                 *");
  printf("\n*                                                 *");
  printf("\n***************************************************\n");

  for(;;)
  {
    if ((FIO2PIN & 0x00000400) == 0)
    {
      printf("\nP2.10 pressed!");
    }

    if ((FIO2PIN & 0x00400000) == 0)
    {
      printf("\nJoystick-CENTER pressed!");
    }

    if ((FIO2PIN & 0x04000000) == 0)
    {
      printf("\nJoystick-RIGHT pressed!");
    }

    if ((FIO2PIN & 0x00800000) == 0)
    {
      printf("\nJoystick-LEFT pressed!");
    }

    if ((FIO2PIN & 0x02000000) == 0)
    {
      printf("\nJoystick-UP pressed!");
    }

    if ((FIO2PIN & 0x08000000) == 0)
    {
      printf("\nJoystick-DOWN pressed!");
    }

    delayMs(10);
  }
}
