/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2008 Embedded Artists AB
 *
 * Description:
 *    Sample application that demonstrates how to use rtc.
 *
 *****************************************************************************/

#include <general.h>
#include <printf_P.h>
#include <ea_init.h>
#include <lpc246x.h>

/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU16 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  T1TCR = 0x02;          //stop and reset timer
  T1PR  = 0x00;          //set prescaler to zero
  T1MR0 = delayInMs * (Fpclk / 1000);
  T1IR  = 0xff;          //reset all interrrupt flags
  T1MCR = 0x04;          //stop timer on match
  T1TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (T1TCR & 0x01)
    ;
}

/*****************************************************************************
 *
 * Description:
 *    The first function to execute 
 *
 ****************************************************************************/
int
main(void)
{
  IODIR1 |= 0x38000000;
  IOCLR1  = 0x38000000;
  PINSEL3 = 0;

  eaInit();
  
  printf("\n***************************************************");
  printf("\n*                                                 *");
  printf("\n* Test program for LPC24xx EDU Board...           *");
  printf("\n* (C) Embedded Artists 2008                       *");
  printf("\n*                                                 *");
  printf("\n* Test - RTC & ALARM                              *");
  printf("\n*                                                 *");
  printf("\n***************************************************\n");

  for(;;)
  {
    static tU32 old_sec = 99;
    
    if ((FIO2PIN & 0x00000400) == 0)
    {
      printf("\nP2.10 pressed!");

      RTC_CCR  = 0x00000012;
      RTC_CCR  = 0x00000010;
      RTC_ILR  = 0x00000007;
      RTC_CIIR = 0x00000000;
      RTC_AMR  = 0xfe; //0x00000000;
      RTC_ALSEC = 45;

      delayMs(100);

      RTC_SEC  = 0;
      RTC_MIN  = 0;
      RTC_HOUR = 0;

      delayMs(100);

      RTC_CCR  = 0x00000011;
    }
    if ((FIO2PIN & 0x00000400) == 0)
      printf("\nP2.10 pressed!");

    if ((FIO2PIN & 0x00400000) == 0)
    {
      printf("\nJoystick-CENTER pressed!");
      IOSET1 = 0x20000000;
    }
    else
      IOCLR1 = 0x20000000;
    if ((FIO2PIN & 0x02000000) == 0)
      RTC_SEC++;
    if ((FIO2PIN & 0x04000000) == 0)
    {
      printf("\nJoystick-RIGHT pressed!");
      IOSET1 = 0x08000000;
    }
    else
      IOCLR1 = 0x08000000;
    if ((FIO2PIN & 0x00800000) == 0)
    {
      printf("\nJoystick-LEFT pressed!");
      IOSET1 = 0x10000000;
    }
    else
      IOCLR1 = 0x10000000;
    if ((FIO2PIN & 0x08000000) == 0)
      RTC_SEC--;

    if (old_sec != RTC_SEC)
    {
      printf("\nMIN:SEC = %d:%d (ILR=%x)", RTC_MIN, RTC_SEC, RTC_ILR);
      old_sec = RTC_SEC;
      
      if (RTC_SEC == 50)
        RTC_ILR = 2;
    }
    delayMs(1);
  }
}
